"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AdmZip = require("adm-zip");
const logging_1 = require("@hibas123/logging");
const path = require("path");
// const Logging = new LoggingBase({
//    name: "@hibas123/epubparser",
//    console: false
// });
const xml2js = require("xml2js");
const jsdom = require("jsdom");
function parseXML(data) {
    return new Promise((yes, no) => xml2js.parseString(data, (err, res) => err ? no(err) : yes(res)));
}
class EpubParser {
    constructor(file, loggingAdapters) {
        this.file = file;
        this.loggingAdapters = loggingAdapters;
        this.logging = new logging_1.LoggingBase({ name: "@hibas123/epubparser", console: false });
        if (loggingAdapters) {
            if (Array.isArray(loggingAdapters)) {
                loggingAdapters.forEach(adapter => this.logging.addAdapter(adapter));
            }
            else {
                this.logging.addAdapter(loggingAdapters);
            }
        }
    }
    async parse() {
        const z = new AdmZip(this.file);
        const getData = (name) => {
            let entry = z.getEntry(name);
            if (!entry) {
                throw new Error("Cannot find entry: " + name);
            }
            return new Promise((yes, no) => entry.getDataAsync(yes));
        };
        const getDataS = (name) => getData(name).then(r => r.toString("utf8"));
        // Logging.debug("Files:", z.getEntries().map(e => e.name).filter(e => !e.startsWith("chapter")));
        const mimetype = await getDataS("mimetype");
        this.logging.debug("MimeType: ", mimetype);
        if (mimetype !== "application/epub+zip")
            throw new Error("Invalid mimetype!");
        const { container } = await parseXML(await getDataS("META-INF/container.xml"));
        this.logging.debug("Getting content.opf");
        const getRelativePathFromFile = (file, newf) => {
            let dir = path.dirname(file);
            let jnd = path.join(dir, newf);
            if (jnd.startsWith("/"))
                jnd = jnd.slice(1);
            return jnd;
        };
        const contentFile = container.rootfiles[0].rootfile.find(e => e.$["media-type"] === "application/oebps-package+xml").$["full-path"];
        this.logging.debug("Content File path:", contentFile);
        const content = await getDataS(contentFile);
        this.logging.debug("Parse XML");
        const parsedContent = await parseXML(content);
        const metadata = parsedContent.package.metadata[0];
        const getMetaValue = (field) => {
            let m = metadata[field];
            if (Array.isArray(m)) {
                m = m[0];
            }
            ;
            if (typeof m === "object") {
                m = m._;
            }
            return m;
        };
        const title = getMetaValue("dc:title");
        const creator = getMetaValue("dc:creator");
        const summary = getMetaValue("dc:description");
        const id = getMetaValue("dc:identifier");
        const manifest = parsedContent.package.manifest[0];
        const items = manifest.item;
        const itemMap = new Map();
        let ncxFile;
        let navFile;
        items.map(e => e.$).forEach(item => {
            const type = item["media-type"];
            const properties = item["properties"];
            const id = item["id"];
            const href = getRelativePathFromFile(contentFile, item["href"]);
            itemMap.set(id, { type, href });
            if (type === "application/x-dtbncx+xml") {
                ncxFile = href;
            }
            else if (properties === "nav") {
                navFile = href;
            }
        });
        let chapterMeta;
        if (ncxFile) {
            this.logging.debug("Loading Chapters via ncx");
            const { ncx } = await parseXML(await getDataS(ncxFile));
            chapterMeta = await Promise.all(ncx.navMap[0].navPoint
                .filter(point => point.$.class === "chapter")
                .map(point => {
                const title = point.navLabel[0].text[0];
                const src = point.content[0].$.src.split("#")[0];
                return { title, src };
            }));
        }
        else if (navFile) {
            this.logging.debug("Loading Chapters via NAV");
            const navContent = await getDataS(navFile);
            const parsed = new jsdom.JSDOM(navContent);
            this.logging.debug("Parsed:", parseXML(navContent));
            const nav = parsed.window.document.querySelector("nav");
            chapterMeta = [];
            nav.querySelectorAll("a").forEach(item => {
                chapterMeta.push({
                    title: item.textContent,
                    src: getRelativePathFromFile(navFile, item.href)
                });
            });
        }
        else {
            this.logging.error("Cannot identify navigation document");
        }
        this.logging.debug("Load ncxFile", ncxFile);
        const chapters = await Promise.all(chapterMeta.map(async ({ title, src }) => {
            this.logging.debug("Load Chapter", title, "from:", src);
            const content = await getDataS(src);
            return {
                title,
                content,
                getFile: (url) => {
                    return getData(path.join(path.dirname(src), url).replace("\\", "/"));
                }
            };
        }));
        return {
            id,
            title,
            creator,
            summary,
            getFile: (href) => getData(href),
            getFileByID: (id) => {
                this.logging.debug("Get File with id:", id);
                let item = itemMap.get(id);
                this.logging.debug(item);
                if (item) {
                    return {
                        getData: () => getData(item.href),
                        type: item.type,
                        href: item.href
                    };
                }
                return undefined;
            },
            chapters
        };
    }
}
exports.default = EpubParser;
//# sourceMappingURL=index.js.map